@@ -736,6 +736,75 @@ static int egalax_init(int fd,
 
 # endif /* SERIO_EGALAX */
 
+#define MTOUCH_CMD1 "\001OI\r"
+#define MTOUCH_CMD2 "\001UT\r"
+
+static int mtouch_read_response(int fd, unsigned char *c, int timeout) {
+	unsigned char *tmp = c;
+	int end = 0;
+
+	while (!end && !readchar(fd, tmp, timeout)) {
+		if (*tmp == '\n' || *tmp == '\r')
+			*tmp = '\0', end = 1;
+		tmp++;
+	}
+
+	return tmp - c;
+}
+
+static int mtouch_init(int fd,
+		       __attribute__ ((unused)) unsigned long *id,
+		       __attribute__ ((unused)) unsigned long *extra) {
+	unsigned char response[128];
+
+	if (write(fd, MTOUCH_CMD1, sizeof(MTOUCH_CMD1) - 1) != sizeof(MTOUCH_CMD1) - 1)
+		return -1;
+	tcdrain(fd);
+
+	if (!mtouch_read_response(fd, response, 200))
+		return -1;
+
+#if 0
+	if (response[0] == 1)
+		printf("MicroTouch Controller ID: %s\n", &response[1]);
+#endif
+
+	if (write(fd, MTOUCH_CMD2, sizeof(MTOUCH_CMD2) - 1) != sizeof(MTOUCH_CMD2) - 1)
+		return -1;
+	tcdrain(fd);
+
+	if (!mtouch_read_response(fd, response, 200))
+		return -1;
+
+#if 0
+	if (response[0] == 1)
+		printf("MicroTouch Unit Type and Status: %s\n", &response[1]);
+#endif
+
+	return 0;
+}
+
+static int elo_init(int fd,
+		    __attribute__ ((unused)) unsigned long *id,
+		    __attribute__ ((unused)) unsigned long *extra) {
+	unsigned char cmd[10] = { 'U', 'i', 0, 0, 0, 0, 0, 0, 0, 0 };
+	unsigned char resp[20];
+	unsigned char *tmp;
+
+	if (write(fd, cmd, 10) != 10)
+		return -1;
+	tcdrain(fd);
+
+	for (tmp = resp; tmp - resp < 20; tmp++)
+		if (readchar(fd, tmp, 100))
+			break;
+
+	if ((tmp - resp) == 20 && resp[0] == 'U' && resp[1] == 'I')
+		return 0;
+
+	return -1;
+}
+
 struct input_types {
 	const char *name;
 	const char *name2;
@@ -750,77 +819,12 @@ struct input_types {
 };
 
 static struct input_types input_types[] = {
-{ "--sunkbd",		"-skb",		"Sun Type 4 and Type 5 keyboards",
-	B1200, CS8,
-	SERIO_SUNKBD,		0x00,	0x00,	1,	NULL },
-{ "--lkkbd",		"-lk",		"DEC LK201 / LK401 keyboards",
-	B4800, CS8|CSTOPB,
-	SERIO_LKKBD,		0x00,	0x00,	1,	NULL },
-{ "--vsxxx-aa",		"-vs",
-			"DEC VSXXX-AA / VSXXX-GA mouse and VSXXX-A tablet",
-	B4800, CS8|CSTOPB|PARENB|PARODD,
-	SERIO_VSXXXAA,		0x00,	0x00,	1,	NULL },
-{ "--spaceorb",		"-orb",		"SpaceOrb 360 / SpaceBall Avenger",
-	B9600, CS8,
-	SERIO_SPACEORB,		0x00,	0x00,	1,	NULL },
-{ "--spaceball",	"-sbl",		"SpaceBall 2003 / 3003 / 4000 FLX",
-	B9600, CS8,
-	SERIO_SPACEBALL,	0x00,	0x00,	0,	spaceball_init },
-{ "--magellan",		"-mag",		"Magellan / SpaceMouse",
-	B9600, CS8 | CSTOPB | CRTSCTS,
-	SERIO_MAGELLAN,		0x00,	0x00,	1,	magellan_init },
-{ "--warrior",		"-war",		"WingMan Warrior",
-	B1200, CS7 | CSTOPB,
-	SERIO_WARRIOR,		0x00,	0x00,	1,	warrior_init },
-{ "--stinger",		"-sting",	"Gravis Stinger",
-	B1200, CS8,
-	SERIO_STINGER,		0x00,	0x00,	1,	stinger_init },
-{ "--mousesystems",	"-msc",		"3-button Mouse Systems mouse",
-	B1200, CS8,
-	SERIO_MSC,		0x00,	0x01,	1,	NULL },
-{ "--sunmouse",		"-sun",		"3-button Sun mouse",
-	B1200, CS8,
-	SERIO_SUN,		0x00,	0x01,	1,	NULL },
 { "--microsoft",	"-bare",	"2-button Microsoft mouse",
 	B1200, CS7,
 	SERIO_MS,		0x00,	0x00,	1,	NULL },
-{ "--mshack",		"-ms",		"3-button mouse in Microsoft mode",
-	B1200, CS7,
-	SERIO_MS,		0x00,	0x01,	1,	NULL },
-{ "--mouseman",		"-mman",	"3-button Logitech / Genius mouse",
-	B1200, CS7,
-	SERIO_MP,		0x00,	0x01,	1,	NULL },
-{ "--mouseman4",	"-mman4",	"4-button Logitech / Genius mouse",
-	B1200, CS7,
-	SERIO_MP,		0x00,	0x03,	1,	NULL },
-{ "--intellimouse",	"-ms3",		"Microsoft IntelliMouse",
-	B1200, CS7,
-	SERIO_MZ,		0x00,	0x11,	1,	NULL },
-{ "--mmwheel",		"-mmw",
-			"Logitech mouse with 4-5 buttons or a wheel",
-	B1200, CS7 | CSTOPB,
-	SERIO_MZP,		0x00,	0x13,	1,	mzp_init },
-{ "--iforce",		"-ifor",	"I-Force joystick or wheel",
-	B38400, CS8,
-	SERIO_IFORCE,		0x00,	0x00,	0,	NULL },
-{ "--newtonkbd",	"-newt",	"Newton keyboard",
-	B9600, CS8,
-	SERIO_NEWTON,		0x00,	0x00,	1,	newton_init },
-{ "--h3600ts",		"-ipaq",	"Ipaq h3600 touchscreen",
-	B115200, CS8,
-	SERIO_H3600,		0x00,	0x00,	0,	NULL },
-{ "--stowawaykbd",	"-ipaqkbd",	"Stowaway keyboard",
-	B115200, CS8,
-	SERIO_STOWAWAY,		0x00,	0x00,	1,	NULL },
-{ "--ps2serkbd",	"-ps2ser",	"PS/2 via serial keyboard",
-	B1200, CS8,
-	SERIO_PS2SER,		0x00,	0x00,	1,	NULL },
-{ "--twiddler",		"-twid",	"Handykey Twiddler chording keyboard",
+{ "--dump",		"-dump",	"Just enable device",
 	B2400, CS8,
-	SERIO_TWIDKBD,		0x00,	0x00,	0,	twiddler_init },
-{ "--twiddler-joy",	"-twidjoy",	"Handykey Twiddler used as a joystick",
-	B2400, CS8,
-	SERIO_TWIDJOY,		0x00,	0x00,	0,	twiddler_init },
+	0,			0x00,	0x00,	0,	dump_init },
 #ifdef SERIO_EGALAX
 { "--eetiegalax",	"-eeti",	"EETI eGalaxTouch",
 	B9600, CS8,
@@ -828,80 +832,148 @@ static struct input_types input_types[] = {
 #endif
 { "--elotouch",		"-elo",		"ELO touchscreen, 10-byte mode",
 	B9600, CS8,
-	SERIO_ELO,		0x00,	0x00,	0,	NULL },
-{ "--elo4002",		"-elo6b",	"ELO touchscreen, 6-byte mode",
-	B9600, CS8 | CRTSCTS,
-	SERIO_ELO,		0x01,	0x00,	0,	NULL },
-{ "--elo271-140",	"-elo4b",	"ELO touchscreen, 4-byte mode",
-	B9600, CS8 | CRTSCTS,
-	SERIO_ELO,		0x02,	0x00,	0,	NULL },
+	SERIO_ELO,		0x00,	0x00,	0,	elo_init },
 { "--elo261-280",	"-elo3b",	"ELO Touchscreen, 3-byte mode",
 	B9600, CS8 | CRTSCTS,
 	SERIO_ELO,		0x03,	0x00,	0,	NULL },
-{ "--hampshire",   "-ham",         "Hampshire touchscreen",
-	B9600, CS8,
-	SERIO_HAMPSHIRE,	0x00,   0x00,   0,  NULL },
-{ "--mtouch",		"-mtouch",	"MicroTouch (3M) touchscreen",
+{ "--elo271-140",	"-elo4b",	"ELO touchscreen, 4-byte mode",
 	B9600, CS8 | CRTSCTS,
-	SERIO_MICROTOUCH,	0x00,	0x00,	0,	NULL },
-#ifdef SERIO_TSC40
-{ "--tsc",		"-tsc",		"TSC-10/25/40 serial touchscreen",
-	B9600, CS8,
-	SERIO_TSC40,		0x00,	0x00,	0,	tsc40_init },
-#endif
-{ "--touchit213",	"-t213",	"Sahara Touch-iT213 Tablet PC",
-	B9600, CS8,
-	SERIO_TOUCHIT213,	0x00,	0x00,	0,	t213_init },
-{ "--touchright",	"-tr",	"Touchright serial touchscreen",
+	SERIO_ELO,		0x02,	0x00,	0,	NULL },
+{ "--elo4002",		"-elo6b",	"ELO touchscreen, 6-byte mode",
 	B9600, CS8 | CRTSCTS,
-	SERIO_TOUCHRIGHT,	0x00,	0x00,	0,	NULL },
-{ "--touchwin",		"-tw",	"Touchwindow serial touchscreen",
-	B4800, CS8 | CRTSCTS,
-	SERIO_TOUCHWIN,		0x00,	0x00,	0,	NULL },
-{ "--penmount9000",		"-pm9k",	"PenMount 9000 touchscreen",
-	B19200, CS8,
-	SERIO_PENMOUNT,		0x00,	0x00,	0,	NULL },
-{ "--penmount6000",		"-pm6k",	"PenMount 6000 touchscreen",
-	B19200, CS8,
-	SERIO_PENMOUNT,		0x01,	0x00,	0,	pm6k_init },
-{ "--penmount3000",		"-pm3k",	"PenMount 3000 touchscreen",
-	B38400, CS8,
-	SERIO_PENMOUNT,		0x02,	0x00,	0,	NULL },
-{ "--penmount6250",		"-pmm1",	"PenMount 6250 touchscreen",
-	B19200, CS8,
-	SERIO_PENMOUNT,		0x03,	0x00,	0,	NULL },
-{ "--fujitsu",		"-fjt",	"Fujitsu serial touchscreen",
-	B9600, CS8,
-	SERIO_FUJITSU,		0x00,	0x00,	1,	fujitsu_init },
-{ "--ps2mult",	"-ps2m",	"PS/2 serial multiplexer",
-	B57600, CS8,
-	SERIO_PS2MULT,		0x00,	0x00,	1,	NULL },
-{ "--zhen-hua",		"-zhen",	"Zhen Hua 5-byte protocol",
-	B19200, CS8,
-	SERIO_ZHENHUA,		0x00,	0x00,	0,	zhenhua_init },
+	SERIO_ELO,		0x01,	0x00,	0,	NULL },
 { "--easypen",		"-ep",		"Genius EasyPen 3x4 tablet",
 	B9600, CS8|CREAD|CLOCAL|HUPCL|PARENB|PARODD,
 	SERIO_EASYPEN,		0x00,	0x00,	0,	easypen_init },
+{ "--fujitsu",		"-fjt",		"Fujitsu serial touchscreen",
+	B9600, CS8,
+	SERIO_FUJITSU,		0x00,	0x00,	1,	fujitsu_init },
+{ "--fsia6b",		"-fsia6b",	"FS-iA6B RC Receiver",
+	B115200, CS8,
+	SERIO_FSIA6B,		0x00,	0x00,	0,	NULL },
+{ "--hampshire",	"-ham",		"Hampshire touchscreen",
+	B9600, CS8,
+	SERIO_HAMPSHIRE,	0x00,   0x00,   0,	NULL },
+{ "--iforce",		"-ifor",	"I-Force joystick or wheel",
+	B38400, CS8,
+	SERIO_IFORCE,		0x00,	0x00,	0,	NULL },
+{ "--h3600ts",		"-ipaq",	"Ipaq h3600 touchscreen",
+	B115200, CS8,
+	SERIO_H3600,		0x00,	0x00,	0,	NULL },
+{ "--stowawaykbd",	"-ipaqkbd",	"Stowaway keyboard",
+	B115200, CS8,
+	SERIO_STOWAWAY,		0x00,	0x00,	1,	NULL },
+{ "--lkkbd",		"-lk",		"DEC LK201 / LK401 keyboards",
+	B4800, CS8|CSTOPB,
+	SERIO_LKKBD,		0x00,	0x00,	1,	NULL },
+{ "--magellan",		"-mag",		"Magellan / SpaceMouse",
+	B9600, CS8 | CSTOPB | CRTSCTS,
+	SERIO_MAGELLAN,		0x00,	0x00,	1,	magellan_init },
+{ "--mouseman",		"-mman",	"3-button Logitech / Genius mouse",
+	B1200, CS7,
+	SERIO_MP,		0x00,	0x01,	1,	NULL },
+{ "--mouseman4",	"-mman4",	"4-button Logitech / Genius mouse",
+	B1200, CS7,
+	SERIO_MP,		0x00,	0x03,	1,	NULL },
+{ "--mmwheel",		"-mmw",
+			"Logitech mouse with 4-5 buttons or a wheel",
+	B1200, CS7 | CSTOPB,
+	SERIO_MZP,		0x00,	0x13,	1,	mzp_init },
+{ "--mshack",		"-ms",		"3-button mouse in Microsoft mode",
+	B1200, CS7,
+	SERIO_MS,		0x00,	0x01,	1,	NULL },
+{ "--intellimouse",	"-ms3",		"Microsoft IntelliMouse",
+	B1200, CS7,
+	SERIO_MZ,		0x00,	0x11,	1,	NULL },
+{ "--mousesystems",	"-msc",		"3-button Mouse Systems mouse",
+	B1200, CS8,
+	SERIO_MSC,		0x00,	0x01,	1,	NULL },
+{ "--mtouch",		"-mtouch",	"MicroTouch (3M) touchscreen",
+	B9600, CS8 | CRTSCTS,
+	SERIO_MICROTOUCH,	0x00,	0x00,	0,	mtouch_init },
+{ "--newtonkbd",	"-newt",	"Newton keyboard",
+	B9600, CS8,
+	SERIO_NEWTON,		0x00,	0x00,	1,	newton_init },
+{ "--spaceorb",		"-orb",		"SpaceOrb 360 / SpaceBall Avenger",
+	B9600, CS8,
+	SERIO_SPACEORB,		0x00,	0x00,	1,	NULL },
+{ "--penmount3000",	"-pm3k",	"PenMount 3000 touchscreen",
+	B38400, CS8,
+	SERIO_PENMOUNT,		0x02,	0x00,	0,	NULL },
+{ "--penmount6000",	"-pm6k",	"PenMount 6000 touchscreen",
+	B19200, CS8,
+	SERIO_PENMOUNT,		0x01,	0x00,	0,	pm6k_init },
+{ "--penmount9000",	"-pm9k",	"PenMount 9000 touchscreen",
+	B19200, CS8,
+	SERIO_PENMOUNT,		0x00,	0x00,	0,	NULL },
+{ "--penmount6250",	"-pmm1",	"PenMount 6250 touchscreen",
+	B19200, CS8,
+	SERIO_PENMOUNT,		0x03,	0x00,	0,	NULL },
+{ "--ps2mult",		"-ps2m",	"PS/2 serial multiplexer",
+	B57600, CS8,
+	SERIO_PS2MULT,		0x00,	0x00,	1,	NULL },
+{ "--ps2serkbd",	"-ps2ser",	"PS/2 via serial keyboard",
+	B1200, CS8,
+	SERIO_PS2SER,		0x00,	0x00,	1,	NULL },
+{ "--pulse8-cec",	"-pulse8-cec",	"Pulse Eight HDMI CEC dongle",
+	B9600, CS8,
+	SERIO_PULSE8_CEC,	0x00,	0x00,	0,	NULL },
+{ "--rainshadow-cec",	"-rainshadow-cec", "RainShadow Tech HDMI CEC dongle",
+	B9600, CS8,
+	SERIO_RAINSHADOW_CEC,	0x00,	0x00,	0,	NULL },
+{ "--spaceball",	"-sbl",		"SpaceBall 2003 / 3003 / 4000 FLX",
+	B9600, CS8,
+	SERIO_SPACEBALL,	0x00,	0x00,	0,	spaceball_init },
+{ "--sunkbd",		"-skb",		"Sun Type 4 and Type 5 keyboards",
+	B1200, CS8,
+	SERIO_SUNKBD,		0x00,	0x00,	1,	NULL },
+{ "--stinger",		"-sting",	"Gravis Stinger",
+	B1200, CS8,
+	SERIO_STINGER,		0x00,	0x00,	1,	stinger_init },
+{ "--sunmouse",		"-sun",		"3-button Sun mouse",
+	B1200, CS8,
+	SERIO_SUN,		0x00,	0x01,	1,	NULL },
+{ "--touchit213",	"-t213",	"Sahara Touch-iT213 Tablet PC",
+	B9600, CS8,
+	SERIO_TOUCHIT213,	0x00,	0x00,	0,	t213_init },
 #ifdef SERIO_TAOSEVM
 { "--taos-evm",		"-taos",	"TAOS evaluation module",
 	B1200, CS8,
 	SERIO_TAOSEVM,		0,	0,	0,	NULL },
 #endif
-{ "--dump",		"-dump",	"Just enable device",
+{ "--touchright",	"-tr",		"Touchright serial touchscreen",
+	B9600, CS8 | CRTSCTS,
+	SERIO_TOUCHRIGHT,	0x00,	0x00,	0,	NULL },
+#ifdef SERIO_TSC40
+{ "--tsc",		"-tsc",		"TSC-10/25/40 serial touchscreen",
+	B9600, CS8,
+	SERIO_TSC40,		0x00,	0x00,	0,	tsc40_init },
+#endif
+{ "--touchwin",		"-tw",		"Touchwindow serial touchscreen",
+	B4800, CS8 | CRTSCTS,
+	SERIO_TOUCHWIN,		0x00,	0x00,	0,	NULL },
+{ "--twiddler",		"-twid",	"Handykey Twiddler chording keyboard",
 	B2400, CS8,
-	0,			0x00,	0x00,	0,	dump_init },
+	SERIO_TWIDKBD,		0x00,	0x00,	0,	twiddler_init },
+{ "--twiddler-joy",	"-twidjoy",	"Handykey Twiddler used as a joystick",
+	B2400, CS8,
+	SERIO_TWIDJOY,		0x00,	0x00,	0,	twiddler_init },
+{ "--vsxxx-aa",		"-vs",
+			"DEC VSXXX-AA / VSXXX-GA mouse and VSXXX-A tablet",
+	B4800, CS8|CSTOPB|PARENB|PARODD,
+	SERIO_VSXXXAA,		0x00,	0x00,	1,	NULL },
 { "--w8001",		"-w8001",	"Wacom W8001",
 	B38400, CS8,
 	SERIO_W8001,		0x00,	0x00,	0,	NULL },
 { "--wacom_iv",		"-wacom_iv",	"Wacom protocol IV tablet",
 	B9600, CS8 | CRTSCTS,
 	SERIO_WACOM_IV,		0x00,	0x00,	0,	wacom_iv_init },
-{ "--pulse8-cec",		"-pulse8-cec",	"Pulse Eight HDMI CEC dongle",
-	B9600, CS8,
-	SERIO_PULSE8_CEC,		0x00,	0x00,	0,	NULL },
-{ "--rainshadow-cec",		"-rainshadow-cec",	"RainShadow Tech HDMI CEC dongle",
-	B9600, CS8,
-	SERIO_RAINSHADOW_CEC,		0x00,	0x00,	0,	NULL },
+{ "--warrior",		"-war",		"WingMan Warrior",
+	B1200, CS7 | CSTOPB,
+	SERIO_WARRIOR,		0x00,	0x00,	1,	warrior_init },
+{ "--zhen-hua",		"-zhen",	"Zhen Hua 5-byte protocol",
+	B19200, CS8,
+	SERIO_ZHENHUA,		0x00,	0x00,	0,	zhenhua_init },
 { NULL, NULL, NULL, 0, 0, 0, 0, 0, 0, NULL }
 };
 
@@ -910,7 +982,12 @@ static void show_help(void)
 	struct input_types *type;
 
 	puts("");
-	puts("Usage: inputattach [--daemon] [--baud <baud>] [--always] [--noinit] <mode> <device>");
+	puts("Usage: inputattach [--daemon] [--baud <baud>] [--[no-]crtscts] [--always] [--noinit] <mode> <device> [...]");
+	puts("Multiple mode / device pairs can be specified if the touchscreens");
+	puts("can be probed properly.");
+	puts("");
+	puts("Options --baud <baud>, --[no-]crtscts, --always and --noinit can appear");
+	puts("before <mode> or between <mode> and <device>.");
 	puts("");
 	puts("Modes:");
 
@@ -924,37 +1001,76 @@ static void show_help(void)
 /* palmed wisdom from http://stackoverflow.com/questions/1674162/ */
 #define RETRY_ERROR(x) (x == EAGAIN || x == EWOULDBLOCK || x == EINTR)
 
+#define MAX_DEVS (4)
+
 int main(int argc, char **argv)
 {
 	unsigned long devt;
 	int ldisc;
-	struct input_types *type = NULL;
-	const char *device = NULL;
+	int ndevs = 0;
+	struct input_types *type[MAX_DEVS] = { NULL, NULL, NULL, NULL };
+	const char *device[MAX_DEVS] = { NULL, NULL, NULL, NULL };
 	int daemon_mode = 0;
 	int need_device = 0;
 	unsigned long id, extra;
 	int fd;
-	int i;
+	int i, j;
 	unsigned char c;
 	int retval;
-	int baud = -1;
-	int ignore_init_res = 0;
-	int no_init = 0;
+	int baud[MAX_DEVS] = { -1, -1, -1, -1 };
+	int crtscts[MAX_DEVS] = { -1, -1, -1, -1 };
+	int ignore_init_res[MAX_DEVS] = { 0, 0, 0, 0 };
+	int no_init[MAX_DEVS] = { 0, 0, 0, 0 };
 
 	for (i = 1; i < argc; i++) {
+		int argidx = ndevs - need_device;
+
 		if (!strcasecmp(argv[i], "--help")) {
 			show_help();
 			return EXIT_SUCCESS;
 		} else if (!strcasecmp(argv[i], "--daemon")) {
 			daemon_mode = 1;
 		} else if (!strcasecmp(argv[i], "--always")) {
-			ignore_init_res = 1;
+			if (argidx >= MAX_DEVS) {
+				fprintf(stderr, "inputattach: Too many options!\n");
+				return EXIT_FAILURE;
+			}
+			ignore_init_res[argidx] = 1;
 		} else if (!strcasecmp(argv[i], "--noinit")) {
-			no_init = 1;
-		} else if (need_device) {
-			device = argv[i];
-			need_device = 0;
+			if (argidx >= MAX_DEVS) {
+				fprintf(stderr, "inputattach: Too many options!\n");
+				return EXIT_FAILURE;
+			}
+			no_init[argidx] = 1;
+		} else if (!strcasecmp(argv[i], "--crtscts")) {
+			if (argidx >= MAX_DEVS) {
+				fprintf(stderr, "inputattach: Too many options!\n");
+				return EXIT_FAILURE;
+			}
+			if (crtscts[argidx] != -1) {
+				fprintf(stderr,
+						"inputattach: duplicate or conflicting "
+						"--crtscts / --no-crtscts options\n");
+				return EXIT_FAILURE;
+			}
+			crtscts[ndevs - need_device] = 1;
+		} else if (!strcasecmp(argv[i], "--no-crtscts")) {
+			if (argidx >= MAX_DEVS) {
+				fprintf(stderr, "inputattach: Too many options!\n");
+				return EXIT_FAILURE;
+			}
+			if (crtscts[argidx] != -1) {
+				fprintf(stderr,
+						"inputattach: duplicate or conflicting "
+						"--crtscts / --no-crtscts options\n");
+				return EXIT_FAILURE;
+			}
+			crtscts[argidx] = 0;
 		} else if (!strcasecmp(argv[i], "--baud")) {
+			if (argidx >= MAX_DEVS) {
+				fprintf(stderr, "inputattach: Too many options!\n");
+				return EXIT_FAILURE;
+			}
 			if (argc <= i + 1) {
 				show_help();
 				fprintf(stderr,
@@ -962,78 +1078,133 @@ int main(int argc, char **argv)
 				return EXIT_FAILURE;
 			}
 
-			baud = atoi(argv[++i]);
-		} else {
-			if (type && type->name) {
-				fprintf(stderr,
-					"inputattach: '%s' - "
-					"only one mode allowed\n", argv[i]);
+			baud[argidx] = atoi(argv[++i]);
+		} else if (need_device) {
+			if (argidx >= MAX_DEVS) {
+				fprintf(stderr, "inputattach: Too many options!\n");
 				return EXIT_FAILURE;
 			}
-			for (type = input_types; type->name; type++) {
-				if (!strcasecmp(argv[i], type->name) ||
-				    !strcasecmp(argv[i], type->name2)) {
+			device[argidx] = argv[i];
+			need_device = 0;
+		} else {
+			struct input_types *tmp;
+
+			if (ndevs >= MAX_DEVS) {
+				fprintf(stderr,
+					"inputattach: '%s' - "
+					"only %d modes allowed\n", argv[i], MAX_DEVS);
+				return EXIT_FAILURE;
+			}
+			for (tmp = input_types; tmp->name; tmp++) {
+				if (!strcasecmp(argv[i], tmp->name) ||
+				    !strcasecmp(argv[i], tmp->name2)) {
 					break;
 				}
 			}
-			if (!type->name) {
+			if (!tmp->name) {
 				fprintf(stderr,
 					"inputattach: invalid mode '%s'\n",
 					argv[i]);
 				return EXIT_FAILURE;
 			}
+			for (j = 0; j < ndevs; j++) {
+				if (type[j] == tmp) {
+					fprintf(stderr,
+						"inputattach: mode '%s' listed twice\n", argv[i]);
+					return EXIT_FAILURE;
+				}
+			}
+			if (ndevs) {
+				if (type[ndevs - 1]->init == NULL) {
+					printf(
+						"inputattach: mode %s cannot be used as "
+						"the previous mode (%s) does not have "
+						"an init function\n",
+						argv[i], type[ndevs - 1]->name);
+					break;
+				} else if (ignore_init_res[ndevs - 1]) {
+					printf(
+						"inputattach: mode %s cannot be used as "
+						"--always is set for the previous mode\n",
+						argv[i]);
+					break;
+				}
+			}
 			need_device = 1;
+			type[ndevs++] = tmp;
 		}
 	}
 
-	if (!type || !type->name) {
+	if (!ndevs) {
 		fprintf(stderr, "inputattach: must specify mode\n");
 		return EXIT_FAILURE;
-        }
+	}
 
 	if (need_device) {
-		fprintf(stderr, "inputattach: must specify device\n");
+		fprintf(stderr,
+				"inputattach: must specify device for mode %s\n",
+				type[ndevs - 1]->name);
 		return EXIT_FAILURE;
 	}
 
-	fd = open(device, O_RDWR | O_NOCTTY | O_NONBLOCK);
-	if (fd < 0) {
-		fprintf(stderr, "inputattach: '%s' - %s\n",
-			device, strerror(errno));
-		return 1;
-	}
+	for (i = 0; i < ndevs; i++) {
+		int flags;
 
-	switch(baud) {
-	case -1: break;
-	case 2400: type->speed = B2400; break;
-	case 4800: type->speed = B4800; break;
-	case 9600: type->speed = B9600; break;
-	case 19200: type->speed = B19200; break;
-	case 38400: type->speed = B38400; break;
-	default:
-		fprintf(stderr, "inputattach: invalid baud rate '%d'\n",
-				baud);
-		return EXIT_FAILURE;
-	}
+		fd = open(device[i], O_RDWR | O_NOCTTY | O_NONBLOCK);
+		if (fd < 0) {
+			fprintf(stderr, "inputattach: '%s' - %s\n",
+				device[i], strerror(errno));
+			return 1;
+		}
 
-	setline(fd, type->flags, type->speed);
+		switch(baud[i]) {
+		case -1: break;
+		case 2400: type[i]->speed = B2400; break;
+		case 4800: type[i]->speed = B4800; break;
+		case 9600: type[i]->speed = B9600; break;
+		case 19200: type[i]->speed = B19200; break;
+		case 38400: type[i]->speed = B38400; break;
+		case 115200: type[i]->speed = B115200; break;
+		default:
+			fprintf(stderr, "inputattach: invalid baud rate '%d'\n",
+					baud[i]);
+			return EXIT_FAILURE;
+		}
 
-	if (type->flush)
-		while (!readchar(fd, &c, 100))
-			/* empty */;
+		flags = type[i]->flags;
+		switch (crtscts[i]) {
+		case 0:
+			flags &= ~CRTSCTS;
+			break;
+		case 1:
+			flags |= CRTSCTS;
+			break;
+		}
+		setline(fd, flags, type[i]->speed);
 
-	id = type->id;
-	extra = type->extra;
+		if (type[i]->flush)
+			while (!readchar(fd, &c, 100))
+				/* empty */;
 
-	if (type->init && !no_init) {
-		if (type->init(fd, &id, &extra)) {
-			if (ignore_init_res) {
-				fprintf(stderr, "inputattach: ignored device initialization failure\n");
-			} else {
-				fprintf(stderr, "inputattach: device initialization failed\n");
-				return EXIT_FAILURE;
+		id = type[i]->id;
+		extra = type[i]->extra;
+
+		if (type[i]->init && !no_init[i]) {
+			if (type[i]->init(fd, &id, &extra)) {
+				if (ignore_init_res[i]) {
+					fprintf(stderr, "inputattach: ignored device initialization failure\n");
+				} else {
+					if (i == ndevs - 1) {
+						fprintf(stderr, "inputattach: device initialization failed\n");
+						return EXIT_FAILURE;
+					} else {
+						close(fd);
+						continue;
+					}
+				}
 			}
 		}
+		break;
 	}
 
 	ldisc = N_MOUSE;
@@ -1042,7 +1213,7 @@ int main(int argc, char **argv)
 		return EXIT_FAILURE;
 	}
 
-	devt = type->type | (id << 8) | (extra << 16);
+	devt = type[i]->type | (id << 8) | (extra << 16);
 
 	if (ioctl(fd, SPIOCSTYPE, &devt) < 0) {
 		fprintf(stderr, "inputattach: can't set device type\n");
